Class: Zstdlib::Inflate
- Defined in:
- ext/zstdlib/ruby/zlib-2.6/zstdlib.c,
ext/zstdlib/ruby/zlib-2.6/zstdlib.c,
ext/zstdlib/ruby/zlib-2.5/zstdlib.c,
ext/zstdlib/ruby/zlib-2.5/zstdlib.c,
ext/zstdlib/ruby/zlib-2.4/zstdlib.c,
ext/zstdlib/ruby/zlib-2.4/zstdlib.c,
ext/zstdlib/ruby/zlib-2.3/zstdlib.c,
ext/zstdlib/ruby/zlib-2.3/zstdlib.c,
ext/zstdlib/ruby/zlib-2.2/zstdlib.c,
ext/zstdlib/ruby/zlib-2.2/zstdlib.c
Overview
Zlib:Inflate is the class for decompressing compressed data. Unlike Zstdlib::Deflate, an instance of this class is not able to duplicate (clone, dup) itself.
Class Method Summary collapse
-
.inflate(src) ⇒ Object
call-seq: Zstdlib.inflate(string) Zstdlib::Inflate.inflate(string).
Instance Method Summary collapse
-
#<<(string) ⇒ Object
Inputs +string+ into the inflate stream just like Zstdlib::Inflate#inflate, but returns the Zstdlib::Inflate object itself.
-
#add_dictionary(dictionary) ⇒ Object
call-seq: add_dictionary(string).
-
#inflate(src) ⇒ Object
call-seq: inflate(deflate_string) -> String inflate(deflate_string) { |chunk| ... } -> nil.
-
#initialize(*args) ⇒ Object
constructor
call-seq: Zstdlib::Inflate.new(window_bits = Zstdlib::MAX_WBITS).
-
#set_dictionary(dic) ⇒ Object
Sets the preset dictionary and returns +string+.
-
#sync(string) ⇒ Object
Inputs +string+ into the end of input buffer and skips data until a full flush point can be found.
-
#sync_point? ⇒ Boolean
Quoted verbatim from original documentation:.
Methods inherited from ZStream
#adler, #avail_in, #avail_out, #avail_out=, #close, #closed?, #data_type, #end, #ended?, #finish, #finished?, #flush_next_in, #flush_next_out, #reset, #stream_end?, #total_in, #total_out
Constructor Details
#initialize(*args) ⇒ Object
call-seq: Zstdlib::Inflate.new(window_bits = Zstdlib::MAX_WBITS)
Creates a new inflate stream for decompression. +window_bits+ sets the size of the history buffer and can have the following values:
0:: Have inflate use the window size from the zlib header of the compressed stream.
(8..15):: Overrides the window size of the inflate header in the compressed stream. The window size must be greater than or equal to the window size of the compressed stream.
Greater than 15:: Add 32 to window_bits to enable zlib and gzip decoding with automatic header detection, or add 16 to decode only the gzip format (a Zstdlib::DataError will be raised for a non-gzip stream).
(-8..-15):: Enables raw deflate mode which will not generate a check value, and will not look for any check values for comparison at the end of the stream.
This is for use with other formats that use the deflate compressed data format such as zip which provide their own check values.
== Example
open "compressed.file" do |compressed_io| zi = Zstdlib::Inflate.new(Zstdlib::MAX_WBITS + 32)
begin
open "uncompressed.file", "w+" do |uncompressed_io|
uncompressed_io << zi.inflate(compressed_io.read)
end
ensure
zi.close
end
end
1900 1901 1902 1903 1904 1905 1906 1907 1908 1909 1910 1911 1912 1913 1914 1915 1916 1917 |
# File 'ext/zstdlib/ruby/zlib-2.6/zstdlib.c', line 1900 static VALUE rb_inflate_initialize(int argc, VALUE *argv, VALUE obj) { struct zstream *z; VALUE wbits; int err; rb_scan_args(argc, argv, "01", &wbits); TypedData_Get_Struct(obj, struct zstream, &zstream_data_type, z); err = inflateInit2(&z->stream, ARG_WBITS(wbits)); if (err != Z_OK) { raise_zlib_error(err, z->stream.msg); } ZSTREAM_READY(z); return obj; } |
Class Method Details
.inflate(src) ⇒ Object
call-seq: Zstdlib.inflate(string) Zstdlib::Inflate.inflate(string)
Decompresses +string+. Raises a Zstdlib::NeedDict exception if a preset dictionary is needed for decompression.
This method is almost equivalent to the following code:
def inflate(string) zstream = Zstdlib::Inflate.new buf = zstream.inflate(string) zstream.finish zstream.close buf end
See also Zstdlib.deflate
1953 1954 1955 1956 1957 1958 1959 1960 1961 1962 1963 1964 1965 1966 1967 1968 1969 1970 1971 1972 1973 1974 |
# File 'ext/zstdlib/ruby/zlib-2.6/zstdlib.c', line 1953 static VALUE rb_inflate_s_inflate(VALUE obj, VALUE src) { struct zstream z; VALUE dst, args[2]; int err; StringValue(src); zstream_init_inflate(&z); err = inflateInit(&z.stream); if (err != Z_OK) { raise_zlib_error(err, z.stream.msg); } ZSTREAM_READY(&z); args[0] = (VALUE)&z; args[1] = src; dst = rb_ensure(inflate_run, (VALUE)args, zstream_end, (VALUE)&z); OBJ_INFECT(dst, src); return dst; } |
Instance Method Details
#<<(string) ⇒ Object
Inputs +string+ into the inflate stream just like Zstdlib::Inflate#inflate, but returns the Zstdlib::Inflate object itself. The output from the stream is preserved in output buffer.
2083 2084 2085 2086 2087 2088 2089 2090 2091 2092 2093 2094 2095 2096 2097 2098 2099 2100 2101 2102 2103 2104 |
# File 'ext/zstdlib/ruby/zlib-2.6/zstdlib.c', line 2083 static VALUE rb_inflate_addstr(VALUE obj, VALUE src) { struct zstream *z = get_zstream(obj); OBJ_INFECT(obj, src); if (ZSTREAM_IS_FINISHED(z)) { if (!NIL_P(src)) { StringValue(src); zstream_append_buffer2(z, src); } } else { do_inflate(z, src); if (ZSTREAM_IS_FINISHED(z)) { zstream_passthrough_input(z); } } return obj; } |
#add_dictionary(dictionary) ⇒ Object
call-seq: add_dictionary(string)
Provide the inflate stream with a dictionary that may be required in the future. Multiple dictionaries may be provided. The inflate stream will automatically choose the correct user-provided dictionary based on the stream's required dictionary.
1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 |
# File 'ext/zstdlib/ruby/zlib-2.6/zstdlib.c', line 1998 static VALUE rb_inflate_add_dictionary(VALUE obj, VALUE dictionary) { VALUE dictionaries = rb_ivar_get(obj, id_dictionaries); VALUE checksum = do_checksum(1, &dictionary, adler32); rb_hash_aset(dictionaries, checksum, dictionary); return obj; } |
#inflate(src) ⇒ Object
call-seq: inflate(deflate_string) -> String inflate(deflate_string) { |chunk| ... } -> nil
Inputs +deflate_string+ into the inflate stream and returns the output from the stream. Calling this method, both the input and the output buffer of the stream are flushed. If string is +nil+, this method finishes the stream, just like Zstdlib::ZStream#finish.
If a block is given consecutive inflated chunks from the +deflate_string+ are yielded to the block and +nil+ is returned.
Raises a Zstdlib::NeedDict exception if a preset dictionary is needed to decompress. Set the dictionary by Zstdlib::Inflate#set_dictionary and then call this method again with an empty string to flush the stream:
inflater = Zstdlib::Inflate.new
begin out = inflater.inflate compressed rescue Zstdlib::NeedDict # ensure the dictionary matches the stream's required dictionary raise unless inflater.adler == Zstdlib.adler32(dictionary)
inflater.set_dictionary dictionary
inflater.inflate ''
end
# ...
inflater.close
See also Zstdlib::Inflate.new
2046 2047 2048 2049 2050 2051 2052 2053 2054 2055 2056 2057 2058 2059 2060 2061 2062 2063 2064 2065 2066 2067 2068 2069 2070 2071 2072 2073 2074 |
# File 'ext/zstdlib/ruby/zlib-2.6/zstdlib.c', line 2046 static VALUE rb_inflate_inflate(VALUE obj, VALUE src) { struct zstream *z = get_zstream(obj); VALUE dst; OBJ_INFECT(obj, src); if (ZSTREAM_IS_FINISHED(z)) { if (NIL_P(src)) { dst = zstream_detach_buffer(z); } else { StringValue(src); zstream_append_buffer2(z, src); dst = rb_str_new(0, 0); OBJ_INFECT(dst, obj); } } else { do_inflate(z, src); dst = zstream_detach_buffer(z); if (ZSTREAM_IS_FINISHED(z)) { zstream_passthrough_input(z); } } return dst; } |
#set_dictionary(dic) ⇒ Object
Sets the preset dictionary and returns +string+. This method is available just only after a Zstdlib::NeedDict exception was raised. See zlib.h for details.
2154 2155 2156 2157 2158 2159 2160 2161 2162 2163 2164 2165 2166 2167 2168 2169 2170 |
# File 'ext/zstdlib/ruby/zlib-2.6/zstdlib.c', line 2154 static VALUE rb_inflate_set_dictionary(VALUE obj, VALUE dic) { struct zstream *z = get_zstream(obj); VALUE src = dic; int err; OBJ_INFECT(obj, dic); StringValue(src); err = inflateSetDictionary(&z->stream, (Bytef*)RSTRING_PTR(src), RSTRING_LENINT(src)); if (err != Z_OK) { raise_zlib_error(err, z->stream.msg); } return dic; } |
#sync(string) ⇒ Object
Inputs +string+ into the end of input buffer and skips data until a full flush point can be found. If the point is found in the buffer, this method flushes the buffer and returns false. Otherwise it returns +true+ and the following data of full flush point is preserved in the buffer.
2114 2115 2116 2117 2118 2119 2120 2121 2122 |
# File 'ext/zstdlib/ruby/zlib-2.6/zstdlib.c', line 2114 static VALUE rb_inflate_sync(VALUE obj, VALUE src) { struct zstream *z = get_zstream(obj); OBJ_INFECT(obj, src); StringValue(src); return zstream_sync(z, (Bytef*)RSTRING_PTR(src), RSTRING_LEN(src)); } |
#sync_point? ⇒ Boolean
Quoted verbatim from original documentation:
What is this?
:)
2131 2132 2133 2134 2135 2136 2137 2138 2139 2140 2141 2142 2143 2144 2145 |
# File 'ext/zstdlib/ruby/zlib-2.6/zstdlib.c', line 2131 static VALUE rb_inflate_sync_point_p(VALUE obj) { struct zstream *z = get_zstream(obj); int err; err = inflateSyncPoint(&z->stream); if (err == 1) { return Qtrue; } if (err != Z_OK) { raise_zlib_error(err, z->stream.msg); } return Qfalse; } |